/* globals CatturaBox, Dialogs */
/* globals ko */

// Questions
// - How to handle LDAP polling in Java plugin?
//   - Dispatch some type of event?
//   - Request 

var _ = require("lodash");
var plugin = ko.computed(function () {
    return CatturaBox.findPlugin("LDAPPlugin");
});

// Constants
var FIELD = {
    USERNAME: "cn",          // defaults to "cn"? "username"?
    PASSWORD: "userPassword" // defaults to "userPassword"
};

var DEFAULT_FILTERS = {
    ANY:          "(objectClass=*)", 
    ANY_USER:     "(objectClass=user)", 
    ANY_ORG_UNIT: "(objectClass=organizationalUnit)"
};

var roles = ko.computed(function () {
    return _(CatturaBox.users.rolesSorted()).map(function (role) {
        return _({
            description: CatturaBox.users.roleDescription(role.name)
        }).extend(role).value();
    }).value();
});

function Query (data)
{
    var t = _(this).extend(data).value();
    
    function toJS ()
    {
        var query = ko.toJS(t);
        delete query.selected;
        return query;
    }
    
    t.role            = ko.observable(ko.unwrap(t.role));
    t.dn              = ko.observable(ko.unwrap(t.dn));
    t.filter          = ko.observable(ko.unwrap(t.filter));
    t.userField       = ko.observable(ko.unwrap(t.userField));
    t.passwordField   = ko.observable(ko.unwrap(t.passwordField));
    t.selected        = ko.observable(false);
    
    t.edit = function edit ()
    {
        addEditQueryDialog(t);
    };
    
    t.test = function test ()
    {
        Dialogs.wait("Testing query...");
        return plugin().services.runQuery({
            query: toJS()
        }).then(function (users) {
            Dialogs.wait.dismiss();
            testQueryDialog(users);
        }).fail(function (error) {
            Dialogs.wait.dismiss();
            CatturaBox.notify("error", "LDAP query failed: " + error.message);
        });
    };
    
    t.save = function save ()
    {
        return plugin().services[t.id ? "updateQuery" : "addQuery"]({
            query: toJS()
        })
        .then(refreshQueries)
        .then(refreshUsers)
        .then(function () {
            CatturaBox.notify("success", "Query (" + t.dn() + ") saved successfully.");
        })
        .fail(function (error) {
            CatturaBox.notify("error", "Failed to save query (" + t.dn() + ") " + error.message);
        });
    };
    
    t.remove = function remove ()
    {
        Dialogs.confirm("Delete Query", "Are you sure you want to delete the query?", function () {
            plugin().services.deleteQuery({ queryId: t.id })
            .then(refreshQueries)
            .then(refreshUsers)
            .then(function () {
                CatturaBox.notify("success", "Query (" + t.dn() + ") removed successfully.");
            });
        }, t.edit.bind(t));
    };
}

// Admin config
var settings = ko.observable();

// Connection status
var status = ko.observable({
    connected: false,
    connectionMessage: "Unknown",
    authenticated: false,
    authenticationMessage: "Unknown"
});

// All queries for each role
var queries = ko.observableArray();
var users   = ko.observableArray();

function refreshAll ()
{
    return refreshSettings().then(refreshStatus).then(refreshQueries).then(refreshUsers);
}

// Refresh queries from plugin
// - Sorted by: role, DN
function refreshQueries ()
{
    return plugin().services.getQueries().then(function (data) {
        queries(
            _(data)
            .map(function (query) {
                return new Query(query);
            })
            .sortBy([
            function (query) {
                return query.role();
            },
            function (query) {
                return query.dn();
            }
            ])
            .value()
        );
    });
}

function refreshSettings ()
{
    return plugin().settings().then(function (data) {
        settings(_(data).toPairs().map(function (pair) {
            var value = pair[1];
            if (value === -1)
            {
                value = undefined;
            }
            
            return [pair[0], ko.observable(value)];
        }).fromPairs().value());
    });
}

function refreshStatus ()
{
    return plugin().services.getConnectionStatus().then(status);
}

function refreshUsers ()
{
    return plugin().services.getUsers().then(users);
}

//function refresh () {
//    services.getQueries()
//    .then(setQueries);
//};

// Refresh queries and return all users
function refresh ()
{
    return plugin().services.refresh().then(users);
}

function saveConfig ()
{
    var $form = $("#ldap-admin-config");
    if (settings().enabled() && !$form.validate())
    {
        return;
    }
    
    return plugin().services["settings.set"]({
        settings: ko.toJS(settings())
    })
    .then(refreshStatus)
    .then(refreshUsers)
    .then(function () {
        CatturaBox.notify("success", "LDAP configuration updated successfully.");
    });    
}

var selectedQueries = ko.computed(function () {
    return _(queries()).filter(function (query) {
        return query.selected();
    }).value();
});

function deleteSelectedQueries ()
{
    Dialogs.confirm("Delete Queries", "Are you sure you want to delete the selected queries?", function () {
        plugin().services.deleteQueries({
            queryIds: _(selectedQueries()).map(function (query) {
                return query.id;
            }).value()
        })
        .then(refreshQueries)
        .then(refreshUsers)
        .then(function () {
            CatturaBox.notify("success", "Selected queries removed successfully.");
        });
    });
}

// Modal
// 03 April 2017 - When the modal is hidden, if it has no parent,
// it will be destroyed (which means we lose knockout bindings). 
// In order to prevent that, we create a parent element which has
// no purpose other than to get around this.
function Modal (options)
{
    var $modal;
    
    function modal ()
    {
        // Delay construction and binding of modal. If this runs before the
        // CatturaBox is bound, Knockout will throw an exception because this
        // modal is already bound to something and attached to the body (so
        // Knockout will try to bind again and fail).
        if (!$modal)
        {
            $modal = $(options.template(modal)).appendTo($("<div>"));
            $(function () {
                $("body").append($modal);
                ko.applyBindings(modal, $modal[0]);
            });
        }
        
        $modal.modal("show");
        
        if (options.onOpen)
        {
            options.onOpen.apply(modal, arguments);
        }
    }
    
    modal.dismiss = function dismiss ()
    {
        if (!$modal)
        {
            return;
        }
        
        $modal.modal("hide");
        
        if (options.onClose)
        {
            options.onClose.apply(modal, arguments);
        }
    };
    
    for (var key in options)
    {
        var value = options[key];
        if (typeof value === "function" && !ko.isObservable(value))
        {
            value = value.bind(modal);
        }
        
        modal[key] = value;
    }
    
    return modal;
}

var addEditQueryDialog = new Modal({
    template: this.templates.editQueryModal,
    
    onOpen: function onOpen (query)
    {
        this.query(new Query(query || {
            filter: this.defaultFilters[0]
        }));
    },

    defaultFilters: _(DEFAULT_FILTERS).values().value(),
    roles: roles,
    query: ko.observable(new Query()),
    settings: settings,
    
    save: function save ()
    {
        var modal = this;
        modal.query().save().then(function () {
            modal.dismiss();
        });
    }
});

var testQueryDialog = new Modal({
    template: this.templates.testQueryModal,
    
    onOpen: function onOpen (users)
    {
        this.users(users);
    },
    
    users: ko.observableArray()
});

// Expose LDAP as an external auth provider
CatturaBox.externalAuth.add("ldap", {
    label: "LDAP",
    
    queries: queries,
    selectedQueries: selectedQueries,
    
    users: users,
    
    status: status,
    settings: settings,
    
    refresh: refresh,
    saveConfig: saveConfig,
    deleteSelectedQueries: deleteSelectedQueries,
    addEditQueryDialog: addEditQueryDialog,
    
    addQuery: addEditQueryDialog.bind(addEditQueryDialog, null)
});

// Events from the recorder
addEventListener("LDAPConnectionStatusEvent", function (event) {
    status(event.status);
    refreshUsers();
});

// UI hooks
var settingsPanelTemplate = this.templates.settingsPanel;
addEventListener("buildExternalAuthTab", function (event) {
    if (event.provider.id !== "ldap")
    {
        return;
    }
    
    event.element.append(settingsPanelTemplate());
});

addEventListener("INIT", function () {
    if (!CatturaBox.user().can("manage users"))
    {
        return;
    }
    
    refreshAll();
});